--
-- Loudmouth
-- Author: FatherJim of Blackhand
-- Email:  praetor@slayernet.org
--

--
-- Set up basic ACE2 library and Fubar data structures and methods.
--

local L = AceLibrary("AceLocale-2.2"):new("Loudmouth")
local tablet = AceLibrary("Tablet-2.0")
Loudmouth = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
Loudmouth:RegisterDB("LoudmouthDB","LoudmouthDBPC")

--
-- Set up the shared SavedVariables.lua defaults.
--

Loudmouth:RegisterDefaults("profile", {
  crits = true,
  fears = true,
  miss = true,
  Channel = 'Say',
  Debug = false,
  _QuoteSet = 1,
  _QuoteEvent = 1,
  rates = {
    ["Crits"] = 25,
    ["Fears"] = 100,
    ["Misses"] = 50,
  },
  quotes = {
    ["Default"] = {
      ["AFK"] = {
        [1] = "Oops, my ethernet cable fell out, and the bits are making a mess on the carpet... brb",
	[2] = "Got a phone call. I won the Lottery. Back after I buy Blizzard to fix a few things...",
      },
      ["Crits"] = {
        [1] = "I just got a critical hit!",
	[2] = "Oops, my weapon just got stuck in you, could you hand that back please?",
      },
      ["Excuses"] = {
      	[1] = "My g/f just called and wife picked up the phone. Sorry, in combat IRL!"
      },
      ["Fears"] = {
        [1] = "Hey mob! I didn't say you could leave yet...",
	[2] = "Oh sure, fight me until you think you're gonna lose, then run away like a little girlie...",
      },
      ["Misses"] = {
        [1] = "Stand still or come closer so I can slap you!",
	[2] = "Oops... I missed.",
      },
    },
  },
})

--
-- Set up the internal variables, menus and structures that aren't user data
--

Loudmouth.hideMenuTitle = true
Loudmouth.hasIcon = true
Loudmouth.editing_item=0
local scrollbar = getglobal("LMFrame_QuoteList_ScrollFrame_ScrollBar")
statement = ''

Loudmouth.dropdowns={}
Loudmouth.dropdowns.Lists={};
Loudmouth.dropdowns.Lists["_QuoteSet"] = { index = "_QuoteSet", label = "Quote Set", selected = 2, table = {[1] = "Default", [2] = "Monty Python"} }
Loudmouth.dropdowns.Lists["_QuoteEvent"] = { index = "_QuoteEvent", label = "Event Type", selected = 2, table = { [1] = "AFK", [2] = "Crits", [3] = "Excuses", [4] = "Fears", [5] = "Misses" } }

local options = {
    type = 'group',
    args = {
    	header = {
	    type = 'header',
	    name = 'Loudmouth',
	    icon = "Interface\\AddOns\\Loudmouth\\icon.tga",
	    iconHeight = 16,
	    iconWidth = 16,
	    order = 1,
	},
        --msg = {
	    --type = 'text',
	    --name = L["Message"],
	    --desc = L["Sets the message to be displayed when you enter a new zone."],
	    --usage = L["<your message>"],
	    --get = "GetMessage",
	    --set = "SetMessage",
	--},
	crits = {
	    type = 'toggle',
	    name = 'Crit Hits',
	    desc = 'Will you brag about critical hits',
	    get = function ()
  		return Loudmouth.db.profile.crits
  	    end,
	    set = function ()
	    	Loudmouth:UpdateTooltip()
  		Loudmouth.db.profile.crits = not Loudmouth.db.profile.crits
	    end,
	},
	critrate = {
	    type = 'range',
	    name = 'Crit Rate',
	    desc = 'How often you brag about critical hits',
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        return Loudmouth.db.profile.rates["Crits"]
	    end,
	    set = function (v)
	    	Loudmouth:UpdateTooltip()
	        Loudmouth.db.profile.rates["Crits"] = v
	    end,
	},
	fears = {
	    type = 'toggle',
	    name = 'Fear Effects',
	    desc = 'Fears Blow',
	    get = function ()
	        return Loudmouth.db.profile.fears
	    end,
	    set = function ()
	    	Loudmouth:UpdateTooltip()
	        Loudmouth.db.profile.fears = not Loudmouth.db.profile.fears
	    end,
	},
	fearrate = {
	    type = 'range',
	    name = 'Fear Rate',
	    desc = 'How often you taunt about Feared mobs running away',
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        return Loudmouth.db.profile.rates["Fears"]
	    end,
	    set = function (v)
	    		Loudmouth:UpdateTooltip()
	        	Loudmouth.db.profile.rates["Fears"] = v
	    end,
	},
	miss = {
	    type = 'toggle',
	    name = 'Miss Attacks',
	    desc = 'Attack Misses',
	    get = function ()
	        return Loudmouth.db.profile.miss
	    end,
	    set = function ()
	    		Loudmouth:UpdateTooltip()
	        	Loudmouth.db.profile.miss = not Loudmouth.db.profile.miss
	    end,
	},
	missrate = {
	    type = 'range',
	    name = 'Miss Rate',
	    desc = 'How often you taunt about Missed Attacks',
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        return Loudmouth.db.profile.rates["Misses"]
	    end,
	    set = function (v)
	    	Loudmouth:UpdateTooltip()
	        Loudmouth.db.profile.rates["Misses"] = v
	    end,
	},
	channel = {
	    type = 'text',
	    name = 'Babble channel',
	    desc = 'Babble randomly in channel',
	    get = function()
	        return Loudmouth.db.profile.Channel
	    end,
	    set = function(v)
	        Loudmouth.db.profile.Channel = v
	    end,
	    validate = { "UI", "Say", "Party", "Raid", "Guild", "Officer" },
	},
	afk = {
		type = 'execute',
		name = 'Show AFK message',
		desc = 'Show an AFK message in guild chat, and raid or party',
		func = function()
		  Loudmouth:ShowAFK()
		end,
	},
	excuse = {
		type = 'execute',
		name = 'Show excuse',
		desc = 'Show an excuse to Guild and default babble channel',
		func = function()
		  Loudmouth:ShowExcuse()
		end,
	},
	gui = {
		type = 'execute',
		name = 'Show GUI',
		desc = 'Display Gui Frame',
		func = function()
		  Loudmouth:ShowGUI()
		end,
	},
	--show = {
	    --type = 'execute',
	    --name = 'Show GUI',
	    --desc = 'Display the GUI Frame',
	    --func = Loudmouth:OnClick(),
	--},
	--hide = {
	    --type = 'execute',
	    --name = 'Hide GUI',
	    --desc = 'Close the GUI Frame',
	    --func = "Loudmouth_Frame:Hide()",
	--},
	--showInChat = {
	    --type = 'toggle',
	    --name = L["Show in Chat"],
	    --desc = L["If set, your message will be displayed in the General chat window."],
	    --get = "IsShowInChat",
	    --set = "ToggleShowInChat",
	--},
    },
}

--
-- Set up ACE and Fubar menus
--

Loudmouth:RegisterChatCommand({"/loudmouth","/lm"}, options)
Loudmouth.OnMenuRequest = options

--
-- Fubar Interface handlers
--

function Loudmouth:OnClick()
  -- Left Click on FUBAR
  Loudmouth:ShowGUI()
end

--function Loudmouth:OnMenuRequest()
  --self:AddImpliedMenuOptions()
--end

function Loudmouth:OnTooltipUpdate()
  local cat = tablet:AddCategory(
    'text', "Event Type",
    'columns', 2,
    'child_textR', 1,
    'child_textG', 1,
    'child_textB', 0,
    'child_textR2', 1,
    'child_textG2', 1,
    'child_textB2', 0
  )
  if (self.db.profile.crits) then
    cat:AddLine(
      'text', "Crit Hits",
      'text2', ""
    )
  end
  if (self.db.profile.fears) then
    cat:AddLine(
      'text', "Fear Effects",
      'text2', ""
    )
  end
  if (self.db.profile.miss) then
    cat:AddLine(
      'text', "Missed Attacks",
      'text2', ""
    )
  end
  local cat = tablet:AddCategory(
    'text', "Response Rates",
    'columns', 2,
    'child_textR', 1,
    'child_textG', 1,
    'child_textB', 0,
    'child_textR2', 1,
    'child_textG2', 1,
    'child_textB2', 0
  )
  if (self.db.profile.crits) then
    cat:AddLine(
      'text', "Crit Hits",
      'text2', self.db.profile.rates["Crits"]
    )
  end
  if (self.db.profile.fears) then
    cat:AddLine(
      'text', "Fear Effects",
      'text2', self.db.profile.rates["Fears"]
    )
  end
  if (self.db.profile.fears) then
    cat:AddLine(
      'text', "Missed Attacks",
      'text2', self.db.profile.rates["Misses"]
    )
  end
  local cat = tablet:AddCategory(
    'text', "Configuration",
    'columns', 2,
    'child_textR', 1,
    'child_textG', 1,
    'child_textB', 1,
    'child_textR2', 0,
    'child_textG2', 1,
    'child_textB2', 1
  )
  cat:AddLine(
    'text', "Babble Channel",
    'text2', self.db.profile.Channel
  )
  cat:AddLine(
    'text', "Number of Quotes",
    'text2', self:NumQuotes()
  )
  cat:AddLine(
    'text', "Quote Set in use",
    'text2', self.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
  )
  tablet:SetTitle("Loudmouth")
  tablet:SetHint("Left click to bring up the configuration dialog. Right click to edit options.")
end

--
-- ACE core functions
--

function Loudmouth:OnEnable()
  -- Called when the addon is enabled
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
  self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
  self:RegisterEvent("CHAT_MSG_SYSTEM")
  self:SetDebugging(Loudmouth.db.profile.Debug)
  Loudmouth.current_event_id=self.db.profile["_QuoteEvent"]
  Loudmouth.current_event_name=Loudmouth.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
  Loudmouth.current_set_name=Loudmouth.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
  UIDropDownMenu_SetSelectedID(LMFrame_QuoteSet, Loudmouth.db.profile["_QuoteSet"])
  UIDropDownMenu_SetSelectedID(LMFrame_QuoteEvent, Loudmouth.db.profile["_QuoteEvent"])

  if (self.db.profile.critrate ~= nil)
  then	self.db.profile.rates["Crits"]=self.db.profile.critrate
  			self.db.profile.critrate = nil
  end
  if (self.db.profile.fearrate ~= nil)
  then	self.db.profile.rates["Fears"]=self.db.profile.fearrate
  			self.db.profile.fearrate = nil
  end
  if (self.db.profile.missrate ~= nil)
  then	self.db.profile.rates["Misses"]=self.db.profile.missrate
  			self.db.profile.missrate = nil
  end
end

--
-- Loudmouth UI code
--

function Loudmouth:DropDown_Initialize()
  local info, index, value, key;
  Loudmouth:Debug("Initializing Dropdown "..this:GetName())
  for index, value in pairs(Loudmouth.dropdowns.Lists) do
  	self:Debug("Index is "..index.." Value.index is "..value.index)
	self:Debug(this:GetName().." ".."LMFrame"..value.index.."Button")
  	if (this:GetName() == "LMFrame"..value.index.."Button")
	then
		self:Debug("Inside IF - #value.table is "..#value.table.." maxn(table)"..table.maxn(value.table));
		self:Debug("Item #1 is "..value.table[1])
		local label = getglobal("LMFrame"..index.."_Label")
		label:SetText(value.label)
		self:Debug("Label should be '"..value.label.."'")
		for key = 1, #value.table do
			info = UIDropDownMenu_CreateInfo()
			if (key == self.db.profile[index])
			then	info.checked = true
			else	info.checked = nil
			end
			info.text = value.table[key]
			info.func = function(x) self:DropDown_OnClick(x) end
			info.arg1 = value.index
			Loudmouth:Debug("Adding 'button' "..info.text)
			UIDropDownMenu_AddButton(info)
		end
		break;
	end
  end
end

function Loudmouth:DropDown_OnClick(list)
  local ddl = getglobal("LMFrame"..list)
  UIDropDownMenu_SetSelectedID(ddl, this:GetID())
  local selected = UIDropDownMenu_GetSelectedValue("LMFrame"..list)
			--if (key == Loudmouth.dropdowns.Lists[index].selected)
  Loudmouth:Debug("I just got a dropdown click..."..this.value.." "..this:GetID())
  if (this:GetID() ~= self.db.profile[list])
  then	self:Debug("Invoking ListBox_OnShow")
  	self.db.profile[list] = this:GetID()
	Loudmouth:EditCancel()
	Loudmouth.current_event_id=self.db.profile["_QuoteEvent"]
	Loudmouth.current_event_name=Loudmouth.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
	Loudmouth.current_set_name=Loudmouth.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
	Loudmouth:Rate_Show()
  	self:ListBox_OnShow("LMFrame"..list)
  else
    self.db.profile[list] = this:GetID()
  end
end

function Loudmouth:DropDown_OnShow()
  local eventName = string.sub(this:GetName(),8)

  self:Debug("DropDown_OnShow("..this:GetName()..")")
  self:Debug("Confirm: Eventname is "..eventName)
  self:Debug(" and "..Loudmouth.dropdowns.Lists[eventName].table[self.db.profile[eventName]])
  UIDropDownMenu_SetSelectedID(this,Loudmouth.dropdowns.Lists[eventName].table[self.db.profile[eventName]], false)
end

function Loudmouth:ListBox_OnLoad(ListBox)
  self:Debug()
end

function Loudmouth:ListBox_OnShow(ListBox)
  local i = 4;
  for i = 1, 15 do
    getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText("")
    getglobal("LMFrame_QuoteList_Button"..i..""):Hide()
  end
  LMFrame_QuoteList_ScrollFrame:Hide()
  local event_name = self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
  self:Debug("Selected field is # "..self.db.profile["_QuoteEvent"])
  self:Debug("Selected field name is "..event_name)
  LMFrame_QuoteList_Label:SetText("Quote List - "..#self.db.profile.quotes['Default'][event_name].." quotes");
  for i = 1, #self.db.profile.quotes["Default"][self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]]
  do
  	self:Debug("Quote # "..i)
	if (i <= 15)
	then	getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText(self.db.profile.quotes["Default"][event_name][i])
    		getglobal("LMFrame_QuoteList_Button"..i):Show()
	else	LMFrame_QuoteList_ScrollFrame:Show()
	end
  end
end

function Loudmouth:ListButton_OnClick(button)
  local quotenum=strsub(button:GetName(),strfind(button:GetName(),"n",-3)+1)
  self:Debug("Clicked button #"..quotenum)
  if (tonumber(quotenum) <= #self.db.profile.quotes["Default"][self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]])
  then
    LMFrame_QuoteEdit:SetText(getglobal(button:GetName().."_Text"):GetText())
    self.editing_item=tonumber(quotenum)
    LMFrame_ButtonDelete:Show()
    --LMFrame_ButtonDelete.disabled='false'
    --LMFrame_ButtonCancel:Show()
    LMFrame_QuoteList_Highlight:Show()
    LMFrame_QuoteList_Highlight:ClearAllPoints()
    LMFrame_QuoteList_Highlight:SetPoint("TOPLEFT",button:GetName(), "TOPLEFT", 0, 0)
  else
    self:Print("Not editing that item, it's blank");
  end
end

function Loudmouth:ListButton_HideTooltip(button)
  --self:Debug("Left mouseover from "..button:GetName())
  GameTooltip:Hide()
end

function Loudmouth:ListButton_ShowTooltip(button)
  --self:Debug("Got mouseover from "..button:GetName())
  GameTooltip:SetOwner(this,"ANCHOR_RIGHT")
  GameTooltip:SetText(getglobal(this:GetName().."_Text"):GetText())
end

function Loudmouth:Rate_Edit_Changed(Box)
  if (Box == nil) then return end
  self:Debug(self.current_event_name.." Edit Box Changed to "..Box:GetText())
  self.db.profile.rates[self.current_event_name] = tonumber(Box:GetText())
  getglobal(Box:GetParent():GetName().."_Slider"):SetValue(tonumber(Box:GetText()))
	    		Loudmouth:UpdateTooltip()
end

function Loudmouth:Rate_Show()
  if (Loudmouth.current_event_name == "Excuses" or
      Loudmouth.current_event_name == "AFK")
  then	LMFrame_Rate:Hide()
  else	LMFrame_Rate:Show()
  			LMFrame_Rate_Label:SetText(Loudmouth.current_event_name.." Rate Slider")
			self:Debug(self.current_event_name.." is "..tonumber(self.db.profile.rates[self.current_event_name]))
  			LMFrame_Rate_EditBox:SetText(tonumber(self.db.profile.rates[self.current_event_name]))
  			LMFrame_Rate_SliderLow:SetText("0")
  			LMFrame_Rate_SliderHigh:SetText("100")
  			LMFrame_Rate_Slider:SetMinMaxValues(0,100)
  			LMFrame_Rate_Slider:SetValueStep(1)
  			LMFrame_Rate_Slider:SetValue(tonumber(self.db.profile.rates[self.current_event_name]))
  end
end

function Loudmouth:Rate_Slider_Changed(Slider)
  if (Slider == nil) then return end
  self:Debug(self.current_event_name.." Slider Changed to "..Slider:GetValue())
  self.db.profile.rates[self.current_event_name] = tonumber(Slider:GetValue())
  getglobal(Slider:GetParent():GetName().."_EditBox"):SetText(tonumber(Slider:GetValue()))
	    		Loudmouth:UpdateTooltip()
end

--
-- Loudmouth Logic code
--

function Loudmouth:BeatTheOdds(rate)
  local randomvalue = math.random()

  self:Debug("Random: "..randomvalue.." rate: "..rate)
  if (randomvalue <= (rate/100)) then
    return true
  else
    return false
  end
end

function Loudmouth:EditCancel()
  LMFrame_QuoteEdit:SetText("")
  self.editing_item = 0
  LMFrame_ButtonDelete:Hide()
  --LMFrame_ButtonDelete.disabled='true'
  LMFrame_QuoteList_Highlight:Hide()
  --LMFrame_ButtonCancel:Hide()
end

function Loudmouth:NumQuotes()
  local number = 0

  number = number + #self.db.profile.quotes["Default"]["AFK"]
  number = number + #self.db.profile.quotes["Default"]["Crits"]
  number = number + #self.db.profile.quotes["Default"]["Excuses"]
  number = number + #self.db.profile.quotes["Default"]["Fears"]
  number = number + #self.db.profile.quotes["Default"]["Misses"]
  return number
end

function Loudmouth:QuoteDelete()
end

function Loudmouth:QuoteEdit()
  local quote = LMFrame_QuoteEdit:GetText()
  local event = self.db.profile["_QuoteEvent"]
  local event_name = self.dropdowns.Lists["_QuoteEvent"].table[event]
  local next_index = #self.db.profile.quotes["Default"][event_name]+1

  --self:Debug("self.editing_item is "..self.editing_item)
  if (self.editing_item == 0) then
  	self.editing_item = next_index
	--self:Debug("Choosing next_index... Next index is: "..next_index)
  end
  --self:Debug("Got a string: "..quote)
  --self:Debug("Event # is "..event.." and event_name is "..event_name)
  --self:Debug("self.editing_item is "..self.editing_item)
  if (self.editing_item <= #self.db.profile.quotes["Default"][event_name])
  then	--self:Debug("Replacing '"..self.db.profile.quotes["Default"][event_name][self.editing_item].."' with '"..quote.."'")
  elseif (strlen(quote) == 0)
  then	self:Print("Quote was blank... Skipping insert.")
  else	--self:Debug("Inserting new quote: '"..quote.."'")
  end

  if (self.editing_item > 0 and strlen(quote) > 0)
  then	--getglobal("LMFrame_QuoteList_Button"..self.editing_item.."_Text"):SetText(quote)
  	self.db.profile.quotes["Default"][event_name][self.editing_item] = quote
	--getglobal("LMFrame_QuoteList_Button"..self.editing_item):Show()
	self.editing_item = 0
  end

  LMFrame_QuoteEdit:SetText("")
  Loudmouth:ListBox_OnShow("LMFrame_QuoteList")
end

function Loudmouth:ShowAFK()
  local num_afks = #self.db.profile.quotes["Default"]["AFK"]
  local index = math.random(num_afks)
  statement = self.db.profile.quotes["Default"]["AFK"][index]
  SendChatMessage(statement,"Guild")
  if (GetNumRaidMembers() > 1) then
    SendChatMessage(statement,"Raid")
  elseif (GetNumPartyMembers() >= 1) then
    SendChatMessage(statement,"Party")
  end
end

function Loudmouth:ShowExcuse()
  local num_excuses = #self.db.profile.quotes["Default"]["Excuses"]
  local index = math.random(num_excuses)
  statement = self.db.profile.quotes["Default"]["Excuses"][index]
  self:Spew()
end

function Loudmouth:ShowGUI()
  Loudmouth.db.profile.Debug = self:IsDebugging()
  LMFrame_QuoteList_Label:SetText("Quote List");
  LMFrame_Version:SetText("version "..Loudmouth.version.." "..Loudmouth.date)
  LMFrame:Show();
  Loudmouth.current_event_id=self.db.profile["_QuoteEvent"]
  Loudmouth.current_event_name=Loudmouth.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
  Loudmouth.current_set_name=Loudmouth.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
  --self:ListBox_OnShow(self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]])
end

function Loudmouth:Spew()
  if (statement == '') then return end

  local output = Loudmouth.db.profile.Channel
  
  if (GetNumRaidMembers() == 0 and output == 'Raid') then
    output = 'Party'; self:Debug("Downgrading from Raid to Party")
  end 

  if (GetNumPartyMembers() == 0 and output == 'Party') then
    output = 'Say'; self:Debug("Downgrading from Party to Say")
  end

  if (output == "UI") then
    self:Print(statement)
  else
    SendChatMessage(statement,output)
  end
  --
  -- The following line probably doesn't belong here, I just wanted to save 
  -- this default in a function I'm sure will be called eventually
  --
  Loudmouth.db.profile.Debug = self:IsDebugging()
end

--
-- End of Organization until Event Handlers
--

function Loudmouth:ListBox_OnScroll(scroll)
  if string.find( scroll:GetName(), "LMFrame_QuoteList" ) then
    self:Debug("QuoteList got scrolled? value = "..getglobal(scroll:GetName().."_ScrollBar"):GetValue())
    Loudmouth:Quote_OnScroll(scroll);
  end
end

function Loudmouth:Quote_OnScroll(scroll)
  local n = getglobal(scroll:GetName().."ScrollBar"):GetValue();
  local buttonNo = 1;

  self:Debug("ScrollBar:GetValue() = "..n)

  for i = 1, 15, 1 do
    getglobal("LMFrame_QuoteList_Button"..i.."Text"):SetText("");
  end

  local temp = LM_Quote_Selected - n;
  if temp <= 0 or temp >= 16 then
    getglobal(scroll:GetParent():GetName().."Highlight"):Hide();
  else
    local highlight = getglobal(scroll:GetParent():GetName().."Highlight");
    highlight:Show();
    highlight:ClearAllPoints();
    highlight:SetPoint("TOPLEFT", scroll:GetParent():GetName().."Button"..temp, "TOPLEFT", 0, 0);
  end

  for key, value in LM_SAVE.Quote do
    if n == 0 then
      if buttonNo == 6 then
        getglobal("LM_Quote_ListBoxScrollFrameScrollBarScrollDownButton"):Enable();
	return;
      end
      getglobal("LM_Quote_ListBoxButton"..buttonNo.."Text"):SetText(key);
      buttonNo = buttonNo + 1;
    else
      n = n - 1;
    end
  end
  for key, value in LM_Save.Search do
    if n == 0 then
      if buttonNo == 6 then
        getglobal(scroll:GetName().."ScrollBarScrollDownButton"):Enable();
	return;
      end
      getglobal(scroll:GetParent():GetName().."Button"..buttonNo.."Text"):SetText(key);
      buttonNo = buttonNo + 1;
    else
      n = n - 1;
    end
  end
  getglobal(scroll:GetName().."ScrollBarScrollDownButton"):Disable();
end

--
-- Deprecated Code
--

--function LM_Dropdown_QuoteEvent_Init(level)
  --event_selected = UIDropDownMenu_GetSelectedValue(LMFrame_QuoteEvent)
  --level = level or 1
  --local info = UIDropDownMenu_CreateInfo();
  --Loudmouth:Debug("Initted QuoteEvent Dropdown")
  --info.text = "Crit Hits";
  --info.func = LM_Dropdown_OnClick;
  --info.value = 'crits'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Missed Attacks";
  --info.value = 'miss'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Mob Feared";
  --info.value = 'fear'
  --if (selectedValue == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
--end

--function LM_Dropdown_QuoteSet_Init(level)
  --quoteset_selected = UIDropDownMenu_GetSelectedValue(LMFrame_QuoteSet)
  --local info;
  --level = level or 1
  --Loudmouth:Debug("Initted QuoteSet Dropdown")
  --local info = UIDropDownMenu_CreateInfo();
  --info.text = "Default"
  --info.func = LM_Dropdown_OnClick;
  --info.value = "default"
  --if (quoteset_selected == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
  --info.text = "Mark"
  --info.value = "Mark"
  --if (quoteset_selected == info.value) then info.checked = true else info.checked = nil end
  --UIDropDownMenu_AddButton(info,level);
--end

--function LM_Dropdown_OnClick()
  --local currentDropDown = UIDropDownMenu_GetCurrentDropDown()
--
  --if (strfind(this:GetName(),"LMFrame_QuoteSet")) then
    --UIDropDownMenu_SetSelectedValue(LMFrame_QuoteSet, this.value)
  --end
  --if (strfind(this:GetName(),"LMFrame_QuoteEvent")) then
    --UIDropDownMenu_SetSelectedValue(LMFrame_QuoteEvent, this.value)
  --end
  --Loudmouth:Debug("I just got a dropdown click..."..this:GetName())
--end

--function Loudmouth:GetDropValue(DropDown)
  --local dropvalue = UIDropDownMenu_GetSelectedValue("LMFrame_QuoteSet")
  --self:Debug("Current value of "..DropDown)
  --self:Debug("is "..dropvalue)
--end

--function Loudmouth:LM_Dropdown_OnShow(list)
  --if (list:GetName() == "LMFrame_QuoteSet") then
    --LMFrame_QuoteSet_Label:SetText("Quote Set")
    ----UIDROPDOWNMENU_OPEN_MENU=1;
    --UIDropDownMenu_Initialize(LMFrame_QuoteSet, LM_Dropdown_QuoteSet_Init)
    --UIDropDownMenu_SetWidth(70,LMFrame_QuoteSet)
    --UIDropDownMenu_ClearAll(LMFrame_QuoteSet)
    --UIDropDownMenu_SetSelectedID(LMFrame_QuoteSet,1)
    --self:Debug("QuoteSet Dropdown Shown")
  --elseif (list:GetName() == "LMFrame_QuoteEvent") then
    ----UIDROPDOWNMENU_OPEN_MENU=2;
    --LMFrame_QuoteEvent_Label:SetText("Event")
    --UIDropDownMenu_Initialize(LMFrame_QuoteEvent, LM_Dropdown_QuoteEvent_Init)
    --UIDropDownMenu_SetWidth(90,LMFrame_QuoteEvent)
    --UIDropDownMenu_ClearAll(LMFrame_QuoteEvent)
    --UIDropDownMenu_SetSelectedID(LMFrame_QuoteEvent,1)
    --self:Debug("QuoteEvent Dropdown Shown")
  --else
    --self:Debug("Unknown Dropdown Shown")
  --end
--end

--
-- All Event handlers are located below this comment
--

function Loudmouth:CHAT_MSG_COMBAT_SELF_HITS()
  local you1, you2 = string.find(arg1, "You");
  local crit, crit2 = string.find(arg1, "crit");
  local num_crits = #self.db.profile.quotes["Default"]["Crits"]
  local index = math.random(num_crits)

  if (you1 == 1 and you2 == 3 and crit) then
    local message = "Loudmouth: Critical Hit! "
    self:Debug("Index = "..index.." num_crits = "..num_crits)
    if (self:BeatTheOdds(Loudmouth.db.profile.rates["Crits"])) then
      --statement = "I just got a critical hit!"
      statement = self.db.profile.quotes["Default"]["Crits"][index]
    else
      message = message .. " - skipped"
    end
    UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  else
    UIErrorsFrame:AddMessage("Loudmouth: normal hit",1.0,1.0,1.0,5.0)
    statement = ""
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_COMBAT_SELF_MISSES()
  local message = "Loudmouth: miss"
  local num_misses = #self.db.profile.quotes["Default"]["Misses"]
  local index = math.random(num_misses)

  if (self:BeatTheOdds(Loudmouth.db.profile.rates["Misses"])) then
    self:Debug("Index is "..index)
    statement = self.db.profile.quotes["Default"]["Misses"][index]
  else
    statement = ""
  end
  UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  self:Spew()
end

function Loudmouth:CHAT_MSG_MONSTER_EMOTE()
  local message = "Loudmouth: runaway "..arg2
  local num_fears = #self.db.profile.quotes["Default"]["Fears"]
  local index = math.random(num_fears)
  if (self:BeatTheOdds(Loudmouth.db.profile.rates["Fears"])) then
    statement = self.db.profile.quotes["Default"]["Fears"][index]
  else
    statement = ""
  end
  UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  self:Spew()
end

function Loudmouth:CHAT_MSG_SYSTEM()
  if not string.find(arg1,"You are now AFK") then return end
  --self:Debug(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9)
  local message = "Loudmouth: AFK"..arg1
  UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  self:UnregisterEvent("CHAT_MSG_SYSTEM")
  Loudmouth:ShowAFK()
  self:RegisterEvent("CHAT_MSG_SYSTEM")
end

--
-- Code Snippets and Comments I want to remember are here to the end. 
-- Probably nothing of real interest down here. :)
--

--LMF = CreateFrame("Frame",nil,UIParent)
--LMFt=LMF:CreateTexture(nil,"BACKGROUND")
--LMFt:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Background")
--LMFt:SetAllPoints(LMF)
--LMF.texture=LMFt
--shown = false

  --LMF:SetFrameStrata("BACKGROUND")
  --LMF:SetWidth(100)
  --LMF:SetHeight(100)
  --LMF:SetPoint("CENTER",0,0)
  --LMF:SetMovable(true)
  --if (shown == true) then
    --LMF:Hide()
    --shown = false
  --else
    --LMF:Show()
    --shown = true
  --end


-- UnitSex("Unit") -> 1 = unknown, 2 = male, 3 = female
--Loudmouth.date="12/24/2006"
--Loudmouth.version="0.25"
-- vim: ts=3 ai
